#include <u.h>
#include <libc.h>
#include <auth.h>
#include <mp.h>
#include <libsec.h>
#include <der.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include "ldap.h"

int
bytecmp(Bytes *b, char *s)
{
	int i;

	for(i = 0; s[i] && i < b->len; i++)
		if(b->data[i] != s[i])
			return b->data[i] - s[i];

	if(s[i])
		return s[i];
	if(i < b->len)
		return b->data[i];
	return 0;
}

vlong
byte2num(Bytes *b)
{
	char tmp[64];

	memcpy(tmp, b->data, b->len);
	tmp[b->len] = 0;
	return atoll(tmp);
}

char *
byte2str(Bytes *b)
{
	char *p;

	p = emalloc9p(b->len+1);
	memcpy(p, b->data, b->len);
	return p;
}

long
mswin2epoch(Bytes *b)
{
	uvlong vl;
	char buf[64];

	assert(b->len < sizeof(buf));
	memcpy(buf, b->data, b->len);
	buf[b->len] = 0;

	vl = strtoull(buf, 0, 10);
	if(vl == 0)
		return 0;

	vl = vl / 10000000LL;
	return vl - 11644473600LL;
}

static int
gstr(char **sp, int n)
{
	char buf[5];

	assert(n < sizeof(buf));
	memcpy(buf, *sp, n);
	buf[n] = 0;
	*sp += n;
	return strtol(buf, nil, 10);
}

/* convert 20100406113946.3Z  to secs since 1/1/70 */
long
iso2epoch(Bytes *b)
{
	Tm tm;
	char *s;

	/* e.g.
	 * 20100406113946.3Z
	 */
	if(b->len < 15 || b->len > 22){
		werrstr("bad zulu time format");
		return -1;
	}
	s = (char *)b->data;

	memset(&tm, 0, sizeof(tm));
	tm.year = gstr(&s, 4) - 1900;
	tm.mon = gstr(&s, 2) -1;
	tm.mday = gstr(&s, 2);
	tm.hour = gstr(&s, 2);
	tm.min = gstr(&s, 2);
	tm.sec = gstr(&s, 2);
	
	if(*s == '.')
		strtol(s+1, &s, 10);	/* skip fractional secconds */

	if(*s == 'Z' || *s == 'z')
		tm.tzoff = 0;
	else
	if(*s == '+' || *s == '-')
		tm.tzoff = strtol(s, nil, 10);

	return tm2sec(&tm);
}

