
enum {
	PathRoot = 0,
	PathCtxt = 1,
};

enum {
	ProtoVers = 3,
};

enum {
	AuthSimple = 0,
	AuthSASL = 3,
	AuthntlmsspNegotiate = 10,
	AuthNtlmssp = 11,
};
	
enum {	/* ldap APIs */
	Tbind,
	Rbind ,
	Tunbind,
	Tsearch,
	RsearchEntry,
	Rsearch,
	Tmodify,
	Rmodify,
	Tadd,
	Radd,
	Tdelete,
	Rdelete,
	Tmodify_dn,
	Rmodify_dn,
	Tcompare,
	Rcompare,
	Tabandon,
	Rreference = 19,
	Textended = 23,
	Rintermediate = 25,
};

enum {		/* search scope */
	ScopeBase,						/* exact DN match only */
	ScopeSingle,					/* Hierarchy depth of 1 only */
	ScopeGlobal,					/* Search from DN down hierarchy */
};

enum {		/* alias dereferencing */
	DerefNever,						/* never deref */
	DerefSearches,					/* don't deref base DN, but do deref things found */
	DerefBase,						/* deref the base DN only */
	DerefAlways,					/* deref everything */
};

enum { 
	QueryAnd,
	QueryOr,
	QueryNot,
	QueryEqualTo,
	QuerySubstring,
	QueryGreaterOrEqual,
	QueryLessOrEqual,
	QueryPresent,
	QueryApproxMatch,
	QueryExtensibleMatch,
};

enum {
	SubstrInitial,
	SubstrAny,
	SubstrFinal,
};

enum {
	TypesAndData,
	TypesOnly,
};

typedef struct Entry Entry;
typedef struct Ldap Ldap;
typedef struct Node Node;
typedef struct Srch Srch;
typedef struct Cache Cache;
typedef struct Namval Namval;

#pragma incomplete Cache

struct Ldap {
	char *base;			/* root DN= where the filesystem mounts (< 128 chars) */
	char *user;			/* user name that we logged in as */
	char *netdir;		/* network directory path */
	char *host;			/* remote hostname */
	char *port;			/* remote port name */
	int sec;			/* session is encrypted */
	TLSconn conn;		/* TLS state */
	int maxidle;		/* max idle time before the server hangs up */
	int seqno;			/* request sequence number */
	int fd;				/* network for i/o */
	Cache *cache;		/* cache to reduce traffic */
};

struct Namval {
	Bytes *name;
	Bytes *value;
};

struct Srch {
	int (*object)(void *aux, Bytes *dn);
	int (*attrib)(void *aux, Namval *nv);
	int (*refer)(void *aux, Bytes *url);
	void *aux;
};

extern char *Debug;
extern int Verbose;
extern Srv *fsp;

#pragma	varargck type "B" Bytes *
#pragma	varargck type "L" Namval *
#pragma	varargck type "D" char *
#pragma	varargck type "P" Bytes *
#pragma	varargck type "R" int

///////////////////////////////////

/* cache.c */
int cached_stat(Cache *cache, char *dn, Dir *d);
int cached_readdir(Cache *cache, char *dn, Dir **dirs);
char *cached_walk(Cache *cache, char *dn, char *name, Qid *q);
Cache *cache_fill(Cache *cache, char *dn, int num, Dir *dirs);
int cache_decref(Cache *cache, char *dn);
int cache_incref(Cache *cache, char *dn);

/* date.c */
int bytecmp(Bytes *b, char *s);
vlong byte2num(Bytes *b);
char *byte2str(Bytes *b);
long mswin2epoch(Bytes *b);
/*convert 20100406113946 .3Z to secs since 1 /1 /70 */long iso2epoch(Bytes *b);

/* fmt.c */
int fmt_namevalue(Fmt *f);
int fmt_text(Fmt *f);
int fmt_errstr(Fmt *f);

/* ldap.c */
Elem mkoctstr(char *str);
Ldap *ldap_connect(char *server);
int ldap_bind(Ldap *l, char *user, char *passwd);
int search_results(Elist *payload, Srch *srch);
int ldap_search(Ldap *l, char *dn, Elem attrs, int scope, Elem filter, int results, Srch *srch);
int ldap_cancel(Ldap *l, int id);
int ldap_unbind(Ldap *l);


/* srch.c */
int srch_root_readdir(Ldap *ldap, char *dn, Dir **dirs);
int srch_root_stat(Ldap *ldap, char *dn, Dir *d);
char *srch_root_walk(Ldap *ldap, char *dn, char *name, Qid *q);
char *srch_walk(Ldap *ldap, char *dn, char *name, Qid *q);
int srch_readdir(Ldap *ldap, char *dn, Dir **dirs);
int srch_stat(Ldap *ldap, char *dn, Dir *d);
int srch_read(Ldap *ldap, char *dn, char **buf);
int srch_look(Ldap *ldap, char *dn, char *name);
int srch_config(Ldap *ldap);

/* dns.c */
int dnsRecord(Fmt *f, Bytes *b, int, void *);
