/*
 * https://ldap.com/ldapv3-wire-protocol-reference-bind/
 */

#include <u.h>
#include <libc.h>
#include <auth.h>
#include <mp.h>
#include <libsec.h>
#include <fcall.h>
#include <thread.h>
#include <9p.h>
#include <der.h>
#include <ip.h>
#include "ldap.h"

char *Debug;
int Verbose;

static int Keepalive;

static void
ding(void *u, char *msg)
{
	USED(u);
	if(strstr(msg, "alarm") != nil)
		noted(NCONT);
	noted(NDFLT);
}

static void
keepalive(Ldap *ldap)
{
	Keepalive = rfork(RFPROC|RFMEM|RFNOTEG|RFNAMEG);
	switch(Keepalive){
	case -1:
		sysfatal("cannot fork %r");
	case 0:
		while(1){
			sleep(900 * ldap->maxidle);		/* 90% of the idle timeout on the server */
			ldap_cancel(ldap, 0);			/* NB: 0 is an illegal message ID */				// FIXME: cancel does not seem to keep active directory alive, need to check
		}
		break;
	default:
		break;
	}
}

static void
usage(void)
{
	fprint(2, "usage: %s [-A] [-d name] [-k spec] [-b base] server pattern\n", argv0);
	fprint(2, "       %s -A          anonymous session\n", argv0);
	fprint(2, "       %s -k spec     extra key specifications passed to factotum\n", argv0);
	fprint(2, "       %s -v          verbose chatter\n", argv0);
	exits("usage");
}

void
main(int argc, char *argv[])
{
	Ldap *ldap;
	UserPasswd *up;
	int anonymous;
	char *keyp, *user, *passwd, *svs, *mtpt;

	keyp = "";
	user = "";
	svs = nil;
	mtpt = nil;
	Verbose = 0;

	passwd = "";
	anonymous = 0;
	ARGBEGIN{
	case 'A':
		anonymous = 1;
		break;
	case 'd':
		Debug = EARGF(usage());
		break;
	case 'k':
		keyp = EARGF(usage());
		break;
	case 'm':
		mtpt = EARGF(usage());
		break;
	case 's':
		svs = EARGF(usage());
		break;
	case 'v':
		Verbose++;
		break;
	case 'C':
		chatty9p++;
		break;
	default:
		usage();
		break;
	}ARGEND;

	if(argc < 1)
		usage();

	if(!mtpt && !svs)
		mtpt = "/n/ldap";

	notify(ding);

	fmtinstall('D', dirfmt);
	fmtinstall('M', dirmodefmt);
	fmtinstall('F', fcallfmt);
	fmtinstall('I', eipfmt);
	fmtinstall('V', eipfmt);
	fmtinstall('E', eipfmt);
	fmtinstall('M', eipfmt);
	fmtinstall('B', fmt_text);
	fmtinstall('L', fmt_namevalue);
	fmtinstall('R', fmt_errstr);

	if((ldap = ldap_connect(argv[0])) == nil)
		sysfatal("%s cannot connect, %r", argv[0]);

	if(! anonymous){
//		if(!ldap->sec)
//			sysfatal("will not authenticate without TLS, try tcp!%s!ldaps", ldap->host);

		if((up = auth_getuserpasswd(auth_getkey, "server=%s proto=pass service=%s %s", ldap->host, ldap->port, keyp)) == nil)
			sysfatal("cannot get key - %r");
		user = up->user;
		passwd = up->passwd;
	}

	if(ldap_bind(ldap, user, passwd) != 0)
		sysfatal("ldap_bind failed, %r");

//	srch_config(ldap);				/* get server's state/config info */

	ldap->maxidle = 800;
	ldap->user = user;

//	keepalive(ldap);

	fsp->aux = ldap;
	postmountsrv(fsp, svs, mtpt, MREPL|MCREATE);

	exits(nil);
}
